@@ -4,12 +4,16 @@ that you can implement to get access to miette's (and related library's) full
 reporting and such features.
 */
 
-use std::{fmt::Display, sync::Arc};
+use std::{
+    fmt::{self, Display},
+    fs,
+    panic::Location,
+};
 
 use crate::MietteError;
 
 /**
-Adds rich metadata to your Error that can be used by [DiagnosticReporter] to print
+Adds rich metadata to your Error that can be used by [DiagnosticReportPrinter] to print
 really nice and human-friendly error messages.
 */
 pub trait Diagnostic: std::error::Error {
@@ -20,7 +24,7 @@ pub trait Diagnostic: std::error::Error {
     /// `E0123` or Enums will work just fine.
     fn code<'a>(&'a self) -> Box<dyn Display + 'a>;
 
-    /// Diagnostic severity. This may be used by [DiagnosticReporter]s to change the
+    /// Diagnostic severity. This may be used by [DiagnosticReportPrinter]s to change the
     /// display format of this diagnostic.
     ///
     /// If `None`, reporters should treat this as [Severity::Error]
@@ -36,7 +40,7 @@ pub trait Diagnostic: std::error::Error {
 
     /// Additional contextual snippets. This is typically used for adding
     /// marked-up source file output the way compilers often do.
-    fn snippets(&self) -> Option<Box<dyn Iterator<Item = DiagnosticSnippet>>> {
+    fn snippets(&self) -> Option<Box<dyn Iterator<Item = DiagnosticSnippet> + '_>> {
         None
     }
 }
@@ -63,12 +67,42 @@ impl<T: Diagnostic + Send + Sync + 'static> From<T> for Box<dyn Diagnostic + 'st
     }
 }
 
+/**
+When used with `?`/`From`, this will wrap any Diagnostics and, when
+formatted with `Debug`, will fetch the current [DiagnosticReportPrinter] and
+use it to format the inner [Diagnostic].
+*/
+pub struct DiagnosticReport {
+    diagnostic: Box<dyn Diagnostic + Send + Sync + 'static>,
+}
+
+impl DiagnosticReport {
+    /// Return a reference to the inner [Diagnostic].
+    pub fn inner(&self) -> &(dyn Diagnostic + Send + Sync + 'static) {
+        &*self.diagnostic
+    }
+}
+
+impl std::fmt::Debug for DiagnosticReport {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        crate::get_reporter().debug(&*self.diagnostic, f)
+    }
+}
+
+impl<T: Diagnostic + Send + Sync + 'static> From<T> for DiagnosticReport {
+    fn from(diagnostic: T) -> Self {
+        DiagnosticReport {
+            diagnostic: Box::new(diagnostic),
+        }
+    }
+}
+
 /**
 Protocol for [Diagnostic] handlers, which are responsible for actually printing out Diagnostics.
 
 Blatantly based on [EyreHandler](https://docs.rs/eyre/0.6.5/eyre/trait.EyreHandler.html) (thanks, Jane!)
 */
-pub trait DiagnosticReporter: core::any::Any + Send + Sync {
+pub trait DiagnosticReportPrinter: core::any::Any + Send + Sync {
     /// Define the report format.
     fn debug(
         &self,
@@ -88,7 +122,7 @@ pub trait DiagnosticReporter: core::any::Any + Send + Sync {
 }
 
 /**
-[Diagnostic] severity. Intended to be used by [DiagnosticReporter] to change the
+[Diagnostic] severity. Intended to be used by [DiagnosticReportPrinter]s to change the
 way different Diagnostics are displayed.
 */
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
@@ -112,7 +146,7 @@ If you can read it, you can source it,
 and it's not necessary to read the whole thing--meaning you should be able to
 support Sources which are gigabytes or larger in size.
 */
-pub trait Source: std::fmt::Debug + Send + Sync + 'static {
+pub trait Source: std::fmt::Debug + Send + Sync {
     /// Read the bytes for a specific span from this Source.
     fn read_span<'a>(
         &'a self,
@@ -171,11 +205,11 @@ impl<'a> SpanContents for MietteSpanContents<'a> {
 A snippet from a [Source] to be displayed with a message and possibly some highlights.
  */
 #[derive(Clone, Debug)]
-pub struct DiagnosticSnippet {
+pub struct DiagnosticSnippet<'a> {
     /// Explanation of this specific diagnostic snippet.
-    pub message: Option<String>,
+    pub message: Option<&'a str>,
     /// A [Source] that can be used to read the actual text of a source.
-    pub source: Arc<dyn Source>,
+    pub source: &'a (dyn Source),
     /// The primary [SourceSpan] where this diagnostic is located.
     pub context: SourceSpan,
     /// Additional [SourceSpan]s that mark specific sections of the span, for
@@ -252,6 +286,26 @@ impl<T: AsRef<str>> From<(T, ByteOffset, ByteOffset)> for SourceSpan {
     }
 }
 
+impl From<(SourceOffset, SourceOffset)> for SourceSpan {
+    fn from((start, len): (SourceOffset, SourceOffset)) -> Self {
+        Self {
+            label: None,
+            offset: start,
+            length: len,
+        }
+    }
+}
+
+impl<T: AsRef<str>> From<(T, SourceOffset, SourceOffset)> for SourceSpan {
+    fn from((label, start, len): (T, SourceOffset, SourceOffset)) -> Self {
+        Self {
+            label: Some(label.as_ref().into()),
+            offset: start,
+            length: len,
+        }
+    }
+}
+
 /**
 "Raw" type for the byte offset from the beginning of a [Source].
 */
@@ -268,6 +322,52 @@ impl SourceOffset {
     pub fn offset(&self) -> ByteOffset {
         self.0
     }
+
+    /// Little utility to help convert line/column locations into
+    /// miette-compatible Spans
+    ///
+    /// This function is infallible: Giving an out-of-range line/column pair
+    /// will return the offset of the last byte in the source.
+    pub fn from_location(source: impl AsRef<str>, loc_line: usize, loc_col: usize) -> Self {
+        let mut line = 0usize;
+        let mut col = 0usize;
+        let mut offset = 0usize;
+        for char in source.as_ref().chars() {
+            if char == '\n' {
+                col = 0;
+                line += 1;
+            } else {
+                col += 1;
+            }
+            if line + 1 >= loc_line && col + 1 >= loc_col {
+                break;
+            }
+            offset += char.len_utf8();
+        }
+
+        SourceOffset(offset)
+    }
+
+    /// Returns an offset for the _file_ location of wherever this function is
+    /// called. If you want to get _that_ caller's location, mark this
+    /// function's caller with `#[track_caller]` (and so on and so forth).
+    ///
+    /// Returns both the filename that was given and the offset of the caller
+    /// as a SourceOffset
+    ///
+    /// Keep in mind that this fill only work if the file your Rust source
+    /// file was compiled from is actually available at that location. If
+    /// you're shipping binaries for your application, you'll want to ignore
+    /// the Err case or otherwise report it.
+    #[track_caller]
+    pub fn from_current_location() -> Result<(String, Self), MietteError> {
+        let loc = Location::caller();
+        Ok((
+            loc.file().into(),
+            fs::read_to_string(loc.file())
+                .map(|txt| Self::from_location(&txt, loc.line() as usize, loc.column() as usize))?,
+        ))
+    }
 }
 
 impl From<ByteOffset> for SourceOffset {
