@@ -2,7 +2,10 @@ use std::fmt;
 
 use thiserror::Error;
 
-use crate::Diagnostic;
+use crate::{Diagnostic, DiagnosticReport};
+
+/// Convenience alias. This is intended to be used as the return type for `main()`
+pub type DiagnosticResult<T> = Result<T, DiagnosticReport>;
 
 /// Convenience [Diagnostic] that can be used as an "anonymous" wrapper for
 /// Errors. This is intended to be paired with [IntoDiagnostic].
@@ -10,8 +13,15 @@ use crate::Diagnostic;
 #[error("{}", self.error)]
 pub struct DiagnosticError {
     #[source]
-    pub error: Box<dyn std::error::Error + Send + Sync + 'static>,
-    pub code: String,
+    error: Box<dyn std::error::Error + Send + Sync + 'static>,
+    code: String,
+}
+
+impl DiagnosticError {
+    /// Return a reference to the inner Error type.
+    pub fn inner(&self) -> &(dyn std::error::Error + Send + Sync + 'static) {
+        &*self.error
+    }
 }
 
 impl Diagnostic for DiagnosticError {
@@ -20,9 +30,6 @@ impl Diagnostic for DiagnosticError {
     }
 }
 
-/// Utility Result type for functions that return boxed [Diagnostic]s.
-pub type DiagnosticResult<T> = Result<T, Box<dyn Diagnostic + Send + Sync + 'static>>;
-
 pub trait IntoDiagnostic<T, E> {
     /// Converts [Result]-like types that return regular errors into a
     /// `Result` that returns a [Diagnostic].
