@@ -1,5 +1,3 @@
-use std::sync::Arc;
-
 use miette::{Diagnostic, Severity, SourceSpan};
 use thiserror::Error;
 
@@ -138,25 +136,58 @@ fn list_help() {
 fn fmt_help() {
     #[derive(Debug, Diagnostic, Error)]
     #[error("welp")]
-    #[diagnostic(
-        code(foo::bar::baz),
-        help("{} {}", 1, self.0),
-    )]
+    #[diagnostic(code(foo::bar::baz), help("{} x {0} x {:?}", 1, "2"))]
     struct FooStruct(String);
 
     assert_eq!(
-        "1 hello".to_string(),
+        "1 x hello x \"2\"".to_string(),
         FooStruct("hello".into()).help().unwrap().to_string()
     );
 
     #[derive(Debug, Diagnostic, Error)]
     #[error("welp")]
-    enum FooEnum {
-        #[diagnostic(code(foo::x), help("{} {}", 1, "bar"))]
-        X,
+    #[diagnostic(code(foo::bar::baz), help("{} x {my_field} x {:?}", 1, "2"))]
+    struct BarStruct {
+        my_field: String,
     }
 
-    assert_eq!("1 bar".to_string(), FooEnum::X.help().unwrap().to_string());
+    assert_eq!(
+        "1 x hello x \"2\"".to_string(),
+        BarStruct {
+            my_field: "hello".into()
+        }
+        .help()
+        .unwrap()
+        .to_string()
+    );
+
+    #[derive(Debug, Diagnostic, Error)]
+    #[error("welp")]
+    enum FooEnum {
+        #[diagnostic(code(foo::x), help("{} x {0} x {:?}", 1, "2"))]
+        X(String),
+
+        #[diagnostic(code(foo::x), help("{} x {len} x {:?}", 1, "2"))]
+        Y { len: usize },
+
+        #[diagnostic(code(foo::x), help("{} x {self:?} x {:?}", 1, "2"))]
+        Z,
+    }
+
+    assert_eq!(
+        "1 x bar x \"2\"".to_string(),
+        FooEnum::X("bar".into()).help().unwrap().to_string()
+    );
+
+    assert_eq!(
+        "1 x 10 x \"2\"".to_string(),
+        FooEnum::Y { len: 10 }.help().unwrap().to_string()
+    );
+
+    assert_eq!(
+        "1 x Z x \"2\"".to_string(),
+        FooEnum::Z.help().unwrap().to_string()
+    );
 }
 
 #[test]
@@ -166,11 +197,9 @@ fn test_snippet_named_struct() {
     #[diagnostic(code(foo::bar::baz))]
     struct Foo {
         // The actual "source code" our contexts will be using. This can be
-        // reused by multiple contexts!
-        //
-        // The `Arc` is so you don't have to clone the entire thing into this
-        // Diagnostic. We just need to be able to read it~
-        src: Arc<String>,
+        // reused by multiple contexts, and just needs to implement
+        // miette::Source!
+        src: String,
 
         // The "snippet" span. This is the span that will be displayed to
         // users. It should be a big enough slice of the Source to provide
@@ -197,9 +226,11 @@ fn test_snippet_named_struct() {
         //   | |
         //   | var 1
         #[highlight(snip)]
-        var1: SourceSpan, // label from SourceSpan is used, if any.
+        // label from SourceSpan is used, if any.
+        var1: SourceSpan,
         #[highlight(snip)]
-        var2: SourceSpan,
+        // Anything that's Clone + Into<SourceSpan> can be used here.
+        var2: (String, usize, usize),
 
         // Now with member source names
         filename: String,
@@ -215,7 +246,7 @@ fn test_snippet_unnamed_struct() {
     #[error("welp")]
     #[diagnostic(code(foo::bar::baz))]
     struct Foo(
-        Arc<String>,
+        String,
         #[snippet(0, "hi")] SourceSpan,
         #[highlight(1)] SourceSpan,
         #[highlight(1)] SourceSpan,
@@ -235,12 +266,12 @@ fn test_snippet_enum() {
     enum Foo {
         #[diagnostic(code(foo::a))]
         A {
-            src: Arc<String>,
-            #[snippet(src, "my_snippet.rs", "hi this is where the thing went wrong")]
+            src: String,
+            #[snippet(src, "hi this is where the thing went wrong")]
             snip: SourceSpan,
-            #[highlight(snip, "var 1")]
+            #[highlight(snip)]
             var1: SourceSpan,
-            #[highlight(snip, "var 2")]
+            #[highlight(snip)]
             var2: SourceSpan,
             filename: String,
             second_message: String,
@@ -249,16 +280,16 @@ fn test_snippet_enum() {
         },
         #[diagnostic(code(foo::b))]
         B(
-            Arc<String>,
-            #[snippet(0, "my_snippet.rs", "hi")] SourceSpan,
-            #[highlight(1, "var 1")] SourceSpan,
+            String,
+            #[snippet(0, "hi")] SourceSpan,
+            #[highlight(1)] SourceSpan,
             #[highlight(1, "var 2")] SourceSpan,
             // referenced source name
             String,
             String,
             #[snippet(0, 4, 5)] SourceSpan,
-            #[highlight(6, "var 3")] SourceSpan,
-            #[highlight(6, "var 4")] SourceSpan,
+            #[highlight(6)] SourceSpan,
+            #[highlight(6)] SourceSpan,
         ),
     }
 }
